@@ -458,6 +458,31 @@ def loadtxt(infile):
     for line in f:
         if not line.startswith('#'):
             data.append([x.strip() for x in line.strip().split('\t')])
+
+    # check the data for consistency
+    start = len(data[0])
+    problems = False
+    for i,x in enumerate(data):
+        if len(x) != start:
+            problems = True
+            break
+
+    # if there are inconsistent lines, an integrity check of the data has to be
+    # carried out, ask the user whether this is wanted, or not
+    if problems:
+        answer = raw_input('[!] Input file contains errors. Do you want the errors to be listed separatedly (y/n)? ')
+        if answer == 'y':
+            for i,x in enumerate(data):
+                if len(x) != start:
+                    print("... {0} columns in line {1}, expected {2} ...\
+                            ...".format(
+                                len(x),
+                                i+1,
+                                start
+                                )
+                            )
+        else:
+            pass
     
     return data
 
