@@ -151,7 +151,10 @@ class _Multiple(object):
         """
         
         # store the class-model which is applied in the respective analysis
-        self.model = eval(model)
+        try:
+            self.model = eval(model)
+        except:
+            self.model = model
         
         self.classes = [token[:] for token in self.tokens]
         for i,tokens in enumerate(self.tokens):
@@ -541,7 +544,7 @@ class _Multiple(object):
                         profileB[j-1],
                         gap_weight = gap_weight
                         )
-                tmp[j,i] = score
+                tmp[i,j] = score
 
         alignedA,alignedB,sim = align_pairwise(
                 lstA,
@@ -618,6 +621,9 @@ class _Multiple(object):
                     )
             
             alm_lst.append(alms)
+            #for x in alm_lst[-1]: #_db
+            #    print('\t'.join([self._get(y,'_classes') for y in x])) #_db
+            #print("") #_db
 
         # get the last stage of each alignment process
         alm_lst = alm_lst[-1]
@@ -730,7 +736,7 @@ class _Multiple(object):
         self.params = '_'.join(
                 [
                     'prog',
-                    model,
+                    repr(model),
                     mode,
                     str(gop),
                     '{0:.1f}'.format(gep_scale),
@@ -864,7 +870,7 @@ class _Multiple(object):
         # create a string with the current parameters
         params = [
                     'lib',
-                    model,
+                    repr(model),
                     mode,
                     '',
                     '{0[0]:.1f}x{0[1]:.1f}x{0[2]:.1f}'.format(scale),
