@@ -355,7 +355,10 @@ def _export_score_dict(score_dict):
         outfile.write('\n')
     outfile.close()
 
-def compile_model(model):
+def compile_model(
+        model,
+        path = None
+        ):
     """
     Function compiles customized sound-class models.
 
@@ -365,6 +368,9 @@ def compile_model(model):
     model : str
         A string indicating the name of the model which shall be created.
 
+    path : str
+        A string indication the path where the model-folder is stored.
+
     Notes
     -----
     A model is defined by a folder placed in :file:`data/models` directory of
@@ -435,7 +441,13 @@ def compile_model(model):
      
     print("[i] Compiling model <"+model+">...")
     # get the path to the models
-    path = os.path.split(os.path.abspath(__file__))[0]+'/models/'+model+'/'
+    if not path:
+        path = os.path.split(os.path.abspath(__file__))[0]+'/models/'+model+'/'
+    else:
+        if path.endswith('/'):
+            path = path+model+'/'
+        else:
+            path = path + '/' + model + '/'
     
     # load the sound classes
     sound_classes = _import_sound_classes(path+'converter')
