@@ -45,6 +45,11 @@ class LexStat(object):
         1   hand    hantʰ   1     hæːnd     1   ...
         2   fist    faustʰ  2     fist      2   ...
         ... ...     ...     ...   ...       ... ...  
+
+    .. todo:: csv-documentation
+       
+       Add a description of the current csv-documentation for interaction with
+       qlc-library.
     """
     def __init__(
             self,
@@ -52,13 +57,226 @@ class LexStat(object):
             ):
 
         # store the name of the input file
-        self.infile = infile.split('/')[-1].replace('.lxs','')
-        
+        if infile.endswith('.lxs'):
+            self.infile = os.path.split(infile)[1].replace('.lxs','')
+            self._init_lxs(infile)
+        elif infile.endswith('.csv'):
+            self.infile = os.path.split(infile)[1].replace('.csv','')
+            self._init_csv(infile)
+        else:
+            self.infile = os.path.split(infile)[1]
+            self._init_csv(infile)
+
+    def _init_csv(
+            self,
+            infile
+            ):
+        """
+        Basic routine for input files in csv-format.
+        """
         # load the data
         try:
-            txt = array(loadtxt(infile),dtype="str")
+            txt = array(loadtxt(infile),dtype='str')
         except IOError:
-            txt = array(loadtxt(infile+'.lxs'),dtype="str")
+            txt = array(loadtxt(infile+'.csv'),dtype='str')
+
+        # converter for headlines into internal format
+        converter = {
+                "ipa" : "words",
+                "gloss" : "items",
+                "item" : "items",
+                "prostrings" : "prostrings",
+                "taxon" : "taxa",
+                "language" : "taxa",
+                "languages" : "taxa",
+                "iso" : "taxa",
+                "taxa" : "taxa",
+                "numbers" : "numbers",
+                "tokens" : "tokens",
+                "glossid" : "numbers",
+                "gloss_id" : "numbers",
+                "id" : "ids",
+                "classes" : "classes",
+                "cogs" : "cogs",
+                "cognates" : "cogs",
+                "cognate_id" : "cogs",
+                "cog_id" : "cogs"
+                }
+        # store the index of the columns in the structure variable of the input
+        # file
+        self._ifs = {}
+        for i,x in enumerate(txt[0]):
+            y = x.strip()
+            try:
+                self._ifs[converter[y.lower()]] = (i,x)
+            except KeyError:
+                if y.lower() not in converter.values():
+                    self._ifs[y.lower()] = (i,x)
+                else:
+                    self._ifs[y.lower()+'-x'] = (i,x)
+
+        # check infile for required information
+        required = [
+                ("words","ipa"),
+                ("numbers","gloss_id"),
+                ("taxa","language"),
+                ("ids","ID")
+                ]
+
+        for r in required:
+            if r[0] not in self._ifs.keys():
+                print('[!] Missing "{0}"-column in input file!'.format(r[1]))
+        
+        # store all values of txt in an array
+        data = array(txt[1:])
+        
+        # get the taxa
+        self.taxa = list(
+                set(
+                    [
+                        x.strip().strip('.') for x in data[:,self._ifs['taxa'][0]]
+                        ]
+                        )
+                    )
+        self.width = len(self.taxa)
+        
+        # get the gloss_ids
+        self.numbers = [int(x) for x in data[:,self._ifs['numbers'][0]]]
+
+        # create the items, if they are not in the data
+        if 'items' not in self._ifs:
+            self.items = [str(x) for x in self.numbers]
+        else:
+            self.items = data[:,self._ifs['items'][0]]
+
+        # create idxd first, i.e. a 2d-array in which each id corresponds to the
+        # later position in the two-dimensional array
+        
+        # create a temporary dictionary that groups all language entries according to identical
+        # meanings, and a temporary dictionary which groups all lines by ID
+        tmp,dataById = {},{}
+        
+        for line in data:
+            lng = self.taxa.index(
+                    line[self._ifs['taxa'][0]].strip().strip('.')
+                    )
+            idx = int(line[self._ifs['ids'][0]])
+            wrd = line[self._ifs['words'][0]].strip()
+            gls = int(line[self._ifs['numbers'][0]])
+
+            try:
+                tmp[gls] += [(idx,lng,wrd)]
+            except KeyError:
+                tmp[gls] = [(idx,lng,wrd)]
+
+            dataById[idx] = line
+        
+        # create idxd, the word-array and the array for the ids
+        self.words = [] # array for the words
+        self.idxs = [] # array for the ids
+        self.idxd = {}
+        self.idx = {}
+        self.dataById = dataById # input data in straight format
+        
+        # add first line to dataById
+        self.dataById[0] = txt[0]
+        
+        lines = 0
+        for key,vals in sorted(tmp.items(),key=lambda x: x[0]):
+            # get the maximum number of synonyms in the data
+            m = max([[i[1] for i in vals].count(j) for j in range(self.width)])
+            tmp_words= [['-' for i in range(self.width)] for j in range(m)]
+            tmp_idxs = [[0 for i in range(self.width)] for j in range(m)]
+            
+            # fill in the values for each line
+            for val in vals:
+                k = 0
+                finished = False
+                while not finished:
+                    if tmp_idxs[k][val[1]]:
+                        k += 1
+                        
+                    else:
+                        tmp_idxs[k][val[1]] = val[0]
+                        tmp_words[k][val[1]] = val[2]
+                        self.idxd[val[0]] = (lines+k,val[1])
+                        finished = True
+    
+            # fill in values for idx
+            self.idx[key] = [i+lines for i in range(m)]
+
+            # increase the line-counter
+            lines += m
+            
+            # fill in the values for the words and the ids
+            for w in tmp_words:
+                self.words.append(w)
+            for i in tmp_idxs:
+                self.idxs.append(i)
+
+        # convert words and idxs to arrays
+        self.words = array(self.words)
+        self.idxs = array(self.idxs)
+
+        # create self.cogs, if this is not already there
+        if 'cogs' not in self._ifs:
+            self.cogs = zeros(self.words.shape,dtype='int')
+        else:
+            self.cogs = zeros(self.words.shape,dtype='int')
+            for i in range(self.words.shape[0]):
+                for j in range(self.words.shape[1]):
+                    self.cogs[i][j] = int(
+                            self.dataById[self.idxs[i][j]][self._ifs['cogs'][0]]
+                            )
+
+        # create a specific format string in order to receive taxa of equal
+        # length
+        mtax = max([len(t) for t in self.taxa])
+        self._txf = '{0:.<'+str(mtax)+'}'
+
+        # get the prosodic strings and the context information
+        self.tokens = self.words.copy().tolist()
+        self.prostrings = self.words.copy()
+        for i in range(self.words.shape[0]):
+            for j in range(self.words.shape[1]):
+                if self.words[i][j] != '-':
+                    try:
+                        if 'tokens' in self._ifs:
+                            tokens = unicode(
+                                    dataById[self.idxs[i][j]][self._ifs['tokens'][0]],
+                                    'utf-8'
+                                    ).split(' ')
+                        else:
+                            tokens = ipa2tokens(self.words[i][j])
+                        sonar = [int(x) for x in tokens2class(tokens,art)]
+                        prostring = prosodic_string(sonar)
+                        self.tokens[i][j] = tokens
+                        self.prostrings[i][j] = prostring
+                    except:
+                        print("[!] Some error with string <{0}> (ID: {1}).".format(
+                            self.words[i][j],
+                            self.idxs[i][j]
+                            ))
+            
+        # turn self.tokens into LingpyArray
+        self.tokens = LingpyArray(self.tokens)
+
+        # get height
+        self.height = len(self.idx)
+
+        # create a dictionary in which data created during the calculation can
+        # by stored
+        self.data = {}
+
+    def _init_lxs(
+            self,
+            infile
+            ):
+        """
+        Basic routine for input files in lxs-format.
+        """
+        # load the data
+        txt = array(loadtxt(infile),dtype="str")
 
         # check whether there are cognates in the data
         if 'COG' in txt[0]:
@@ -68,7 +286,7 @@ class LexStat(object):
             self.words = txt[1:,etr_idx]
         else:
             etr_idx = [i for i in range(txt.shape[1]) if txt[0][i].lower() not in
-                    ['number','words']]
+                    ['number','number','words','id']]
             self.words = txt[1:,etr_idx]
             self.cogs = zeros(self.words.shape,dtype='int')
         
@@ -137,6 +355,33 @@ class LexStat(object):
         # by stored
         self.data = {}
 
+        # create the self._ifs file for consistency with new csv input format
+        # operations (also for output, etc.)
+        self._ifs = {
+                "ids":(0,'ID'),
+                "taxa":(1,'Taxon'),
+                "items":(2,'Gloss'),
+                "numbers":(3,"GlossID"),
+                "words":(4,"IPA"),
+                "tokens":(5,"Tokens"),
+                }
+
+        # create the self.dataById dictionary for consistency with new csv
+        # input format
+        self.dataById = {}
+        for i in range(self.idxs.shape[0]):
+            for j in range(self.idxs.shape[1]):
+                if self.idxs[i][j] != 0:
+                    self.dataById[self.idxs[i][j]] = [
+                            str(self.idxs[i][j]),
+                            self.taxa[j],
+                            self.items[i],
+                            str(self.numbers[i]),
+                            self.words[i][j],
+                            ' '.join(self.tokens[i][j]).encode('utf-8')
+                            ]
+        self.dataById[0] = ["ID","Taxon","Gloss","GlossID","IPA","Tokens"]
+
     def _flatten(
             self,
             idx
@@ -263,7 +508,10 @@ class LexStat(object):
         """
         
         # store model and scoring dictionary as attributes
-        self.model = eval(model)
+        try:
+            self.model = eval(model)
+        except:
+            self.model = model
 
         # define the arrays for sound classes and numbers
         self.classes = self.words.copy()
@@ -1319,7 +1567,7 @@ class LexStat(object):
         # into a binary file and loaded, if the calculation already has been
         # carried out before
         vals = {
-                'model':model,
+                'model':repr(model),
                 'merge_vowels':str(merge_vowels),
                 'scale':'-'.join([str(k) for k in scale]),
                 'factor':str(factor),
@@ -1366,7 +1614,7 @@ class LexStat(object):
 
     def output(
             self,
-            fileformat = 'lxs',
+            fileformat = 'csv',
             filename = None
             ):
         """
@@ -1383,10 +1631,11 @@ class LexStat(object):
             * 'msa' -- output of all cognate sets in separate files in
               ``msa``-format, 
             * 'alm' -- output of all cognate sets in one file in ``alm``-format
-              (all data with presumed cognates in aligned format), or
+              (all data with presumed cognates in aligned format), 
             * 'dst' -- the distance matrix in ``dst``-format (input format for
               distance analyses in the Phylip software package (see
-              http://evolution.genetics.washington.edu/phylip/).
+              http://evolution.genetics.washington.edu/phylip/), or
+            * 'csv' -- output in ``csv``-format.
         
         filename : str
             Select a specific name for the outfile, otherwise, the name of
@@ -1456,7 +1705,7 @@ class LexStat(object):
                             out.write('\t'+self.words[i][j])
                         out.write('\n')
                     out.close()
-
+        
         if fileformat == 'star':
             # output the star-files, which can easily be converted into a
             # lexicostatistical list with given cognate judgments (or not)
@@ -1641,3 +1890,24 @@ class LexStat(object):
                 out.write('\n')
 
             out.close()
+
+        if fileformat == 'csv':
+            out = open(outfile,'w')
+            out.write('# '+self.infile+'\n')
+            
+            if sum(self.cogs.flatten()) != 0:
+                out.write('\t'.join(self.dataById[0])+"\tCogID"'\n')
+                for key,value in sorted(
+                        self.dataById.items(),
+                        key=lambda x:x[0]
+                        )[1:]:
+                    out.write(
+                            '\t'.join(value)+'\t'+str(self.cogs[self.idxd[key]])+'\n'
+                            )
+            else:
+                out.write('\t'.join(self.dataById[0])+'\n')
+                for key,value in sorted(self.dataById.items(),key=lambda x:x[0]):
+                    out.write(
+                            '\t'.join(value)+'\n'
+                            )
+            out.close()
